varying highp vec2 Texcoord0;
varying highp vec2 Texcoord1;

uniform sampler2D   TextureSrc;
uniform sampler2D   TextureDst;
uniform highp vec2  ViewportSize;

uniform sampler2D   TextureMask;
uniform bool        HasMask;
uniform bool        RenderToScreen;

uniform highp float Alpha;
uniform highp float Transparency;

highp float Lum(highp vec3 C)
{
    return 0.3 * C.r + 0.59 * C.g + 0.11 * C.b;
}

highp vec3 ClipColor(highp vec3 C)
{
    highp float L = Lum(C);
    
    highp float n = min(min(C.r, C.g), C.b);
    highp float x = max(max(C.r, C.g), C.b);
    
    highp vec3 vL = vec3(L);
    
    highp vec3 _C = C;
    
    if(n < 0.0)
    {
        C = vL + (((C-vL)*L)/(L-n));
    }
    
    if(x > 1.0)
    {
        C = vL + (((C-vL)*(1.0-L))/(x-L));
    }
    
    return _C;
}

highp vec3 SetLum(highp vec3 C, highp float l)
{
    highp float d = l - Lum(C);
    
    highp vec3 _C = vec3(C.r + d, C.g + d, C.b + d);
    
    return ClipColor(_C);
}

void main()
{
    const highp float EPSILON = 1e-5;
    
    highp vec2 Texcoord2 = vec2(gl_FragCoord.x/ViewportSize.x, gl_FragCoord.y/ViewportSize.y);
    
    highp vec4 colorSrc = texture2D(TextureSrc, Texcoord0);
    highp vec4 colorDst = texture2D(TextureDst, Texcoord2);
    
    highp float alpha = colorSrc.a*Alpha;
    
    // modifier based on mask and rendering mode
    highp float modifier = 1.0;
    
    //---------------------------------------------------------------------------
    if(HasMask)
    {
        modifier = texture2D(TextureMask, vec2(Texcoord1.x, 1.0-Texcoord1.y)).r;
        
        if(modifier < EPSILON)
        {
            modifier = Transparency;
        }
    }
    else
    {
        if(Transparency > EPSILON)
        {
            modifier = Transparency;
        }
    }
    //---------------------------------------------------------------------------
    
    alpha = alpha * modifier;
    
    highp vec3  _color;
    highp float _alpha;
    
    //---------------------------------------------------------------
    // final color on screen
    highp vec3 blendColor;
    
    if(RenderToScreen)
    {
        if(colorDst.a == 0.0)
        {
            _color = colorSrc.xyz;
            
            _alpha = alpha;
        }
        else
        {
            blendColor = SetLum(colorDst.xyz, Lum(colorSrc.xyz));
            
            _color = colorSrc.xyz * (1.0 - colorDst.a) + blendColor * colorDst.a;
            
            _alpha = 1.0 - (1.0 - alpha)*(1.0 - colorDst.a);
        }
        
        _color = _color * alpha + colorDst.xyz * (1.0-alpha);
    }
    else
    {
        if(colorDst.a == 0.0)
        {
            _color = colorSrc.xyz;
        }
        else
        {
            //---------------------------------------------------------------
            // compute blend color
            blendColor = SetLum(colorDst.xyz, Lum(colorSrc.xyz));
            //---------------------------------------------------------------
            
            _color = colorSrc.xyz * (1.0 - colorDst.a) + blendColor * colorDst.a;
            
            _color = _color * alpha + colorDst.xyz * (1.0-alpha);
        }
        
        _alpha = 1.0 - (1.0 - alpha)*(1.0 - colorDst.a);
    }
    
    //---------------------------------------------------------------
    
    gl_FragColor = vec4(_color.x,_color.y,_color.z,_alpha);
}
