varying highp vec2 Texcoord0;
varying highp vec2 Texcoord1;

uniform sampler2D   TextureSrc;
uniform sampler2D   TextureDst;
uniform highp vec2  ViewportSize;

uniform sampler2D   TextureMask;
uniform bool        HasMask;
uniform bool        RenderToScreen;

uniform highp float Alpha;
uniform highp float Transparency;

void main()
{
    const highp float EPSILON = 1e-5;
    
    highp vec2 Texcoord2 = vec2(gl_FragCoord.x/ViewportSize.x, gl_FragCoord.y/ViewportSize.y);
    
    highp vec4 colorSrc = texture2D(TextureSrc, Texcoord0);
    highp vec4 colorDst = texture2D(TextureDst, Texcoord2);
    
    highp float alpha = colorSrc.a*Alpha;
    
    // modifier based on mask and rendering mode
    highp float modifier = 1.0;
    
    //---------------------------------------------------------------------------
    if(HasMask)
    {
        modifier = texture2D(TextureMask, vec2(Texcoord1.x, 1.0-Texcoord1.y)).r;
        
        if(modifier < EPSILON)
        {
            modifier = Transparency;
        }
    }
    else
    {
        if(Transparency > EPSILON)
        {
            modifier = Transparency;
        }
    }
    //---------------------------------------------------------------------------
    
    alpha = alpha * modifier;
    
    highp vec3  _color;
    highp float _alpha;
    
    //---------------------------------------------------------------
    // final color on screen
    highp vec3 blendColor;
    
    if(RenderToScreen)
    {
        if(colorDst.a == 0.0)
        {
            _color = colorSrc.xyz;
            
            _alpha = alpha;
        }
        else
        {
            for(int n = 0; n < 3; n++)
            {
                if(colorSrc[n] < 0.5)
                {
                    blendColor[n] = 2.0 * colorSrc[n] * colorDst[n] + colorDst[n]*colorDst[n]*(1.0-2.0*colorSrc[n]);
                }
                else
                {
                    blendColor[n] = 2.0 * colorDst[n] * (1.0-colorSrc[n]) + sqrt(colorDst[n])*(2.0*colorSrc[n]-1.0);
                }
            }
            
            _color = colorSrc.xyz * (1.0 - colorDst.a) + blendColor * colorDst.a;
            
            _alpha = 1.0 - (1.0 - alpha)*(1.0 - colorDst.a);
        }
        
        _color = _color * alpha + colorDst.xyz * (1.0-alpha);
    }
    else
    {
        if(colorDst.a == 0.0)
        {
            _color = colorSrc.xyz;
        }
        else
        {
            //---------------------------------------------------------------
            // compute blend color
            for(int n = 0; n < 3; n++)
            {
                if(colorSrc[n] < 0.5)
                {
                    blendColor[n] = 2.0 * colorSrc[n] * colorDst[n] + colorDst[n]*colorDst[n]*(1.0-2.0*colorSrc[n]);
                }
                else
                {
                    blendColor[n] = 2.0 * colorDst[n] * (1.0-colorSrc[n]) + sqrt(colorDst[n])*(2.0*colorSrc[n]-1.0);
                }
            }
            //---------------------------------------------------------------
            
            _color = colorSrc.xyz * (1.0 - colorDst.a) + blendColor * colorDst.a;
            
            _color = _color * alpha + colorDst.xyz * (1.0-alpha);
        }
        
        _alpha = 1.0 - (1.0 - alpha)*(1.0 - colorDst.a);
    }
    
    //---------------------------------------------------------------
    
    gl_FragColor = vec4(_color.x,_color.y,_color.z,_alpha);
}
