varying highp vec2  Texcoord0;

uniform sampler2D   TextureSrc;
uniform sampler2D   TextureMask;
uniform bool        HasMask;
uniform bool        RenderToScreen;

uniform highp float Alpha;
uniform highp float Transparency;

void main()
{
    const highp float EPSILON = 1e-5;
    
    highp vec4 colorSrc = texture2D(TextureSrc, Texcoord0);
    highp vec4 colorDst = vec4(0,0,0,0);
    
    highp float alpha = colorSrc.a*Alpha;
    
    // modifier based on mask and rendering mode
    highp float modifier = 1.0;
    
    //---------------------------------------------------------------------------
    if(HasMask)
    {
        modifier = texture2D(TextureMask, vec2(Texcoord0.x, 1.0-Texcoord0.y)).r;
        
        if(modifier < EPSILON)
        {
            modifier = Transparency;
        }
    }
    else
    {
        if(Transparency > EPSILON)
        {
            modifier = Transparency;
        }
    }
    //---------------------------------------------------------------------------
    
    alpha = alpha * modifier;
    
    highp vec3  _color;
    highp float _alpha;
    
    //---------------------------------------------------------------
    // final color on screen
    highp vec3 blendColor;
    
    if(RenderToScreen)
    {
        if(colorDst.a == 0.0)
        {
            _color = colorSrc.xyz;
            
            _alpha = alpha;
        }
        else
        {
            blendColor = colorSrc.xyz * colorDst.xyz;
            
            _color = colorSrc.xyz * (1.0 - colorDst.a) + blendColor * colorDst.a;
            
            _alpha = 1.0 - (1.0 - alpha)*(1.0 - colorDst.a);
        }
        
        _color = _color * alpha + colorDst.xyz * (1.0-alpha);
    }
    else
    {
        if(colorDst.a == 0.0)
        {
            _color = colorSrc.xyz;
        }
        else
        {
            //---------------------------------------------------------------
            // compute blend color
            blendColor = colorSrc.xyz * colorDst.xyz;
            //---------------------------------------------------------------
            
            _color = colorSrc.xyz * (1.0 - colorDst.a) + blendColor * colorDst.a;
            
            _color = _color * alpha + colorDst.xyz * (1.0-alpha);
        }
        
        _alpha = 1.0 - (1.0 - alpha)*(1.0 - colorDst.a);
    }
    
    //---------------------------------------------------------------
    
    gl_FragColor = vec4(_color.x,_color.y,_color.z,_alpha);

}
