varying highp vec2  Texcoord0;

uniform highp float Radius;
uniform highp vec2  OutputSize;

uniform sampler2D   Texture;
uniform sampler2D   Mask;

highp float ColorDiffL2(highp vec4 clr0, highp vec4 clr1)
{
    highp float diff = 0.0;
    
    diff+=(clr0.r-clr1.r)*(clr0.r-clr1.r);
    diff+=(clr0.g-clr1.g)*(clr0.g-clr1.g);
    diff+=(clr0.b-clr1.b)*(clr0.b-clr1.b);
    
    diff = diff/3.0;
    
    return diff;
}

void main()
{
    const highp float sigmaColor = 30.0 / 255.0;
    
    // sample central color for comparison
    // WARNING: flip color y axis since we want to make the mask coord consistent with no refinement mask
    highp vec4  centralColor = texture2D(Texture, vec2(Texcoord0.x, 1.0 - Texcoord0.y));
    
    highp float finalMaskValue = 0.0;
    
    highp float totalWeight = 0.0;
    
    highp float varianceDist  = Radius * Radius / 4.5;
    highp float varianceColor = sigmaColor*sigmaColor;
    
    for(highp float y = -Radius; y <= Radius; y++)
    {
        for(highp float x = -Radius; x <= Radius; x++)
        {
            highp vec2 coord = vec2((OutputSize.x - 1.0) * Texcoord0.x + x, (OutputSize.y - 1.0) * Texcoord0.y + y);
            
            // outside the image
            if( coord.x < 0.0 || coord.x >= OutputSize.x || coord.y < 0.0 || coord.y >= OutputSize.y)
            {
                continue;
            }
            
            highp vec2 coord_normalized = vec2( coord.x / (OutputSize.x - 1.0), coord.y / (OutputSize.y - 1.0) );
            
            // WARNING: flip color y axis since we want to make the mask coord consistent with no refinement mask
            highp vec4 color = texture2D(Texture, vec2(coord_normalized.x, 1.0 - coord_normalized.y));
            
            highp float mask = texture2D(Mask, coord_normalized).r;
            
            highp float weightDist  = exp(-(x*x+y*y) / varianceDist);
            
            highp float weightColor = exp(-ColorDiffL2(color, centralColor) / varianceColor);
            
            highp float weight = weightDist * weightColor;
            
            totalWeight += weight;
            
            finalMaskValue += mask * weight;
        }
    }
    
    finalMaskValue = finalMaskValue / totalWeight;
    
    gl_FragColor = vec4(finalMaskValue, finalMaskValue, finalMaskValue, 1.0);
}

