varying highp vec2      Texcoord0;

uniform sampler2D       ImageTexture;
uniform sampler2D       TrimapTexture;
uniform sampler2D       AlphaTexture;

uniform highp vec2      OutputSize;

//uniform mediump vec3 globalFgSample;
//uniform mediump vec3 globalBgSample;
//uniform lowp vec2 globalFgSampleLoc;
//uniform lowp vec2 globalBgSampleLoc;

//output - fg x,y; bg x,y

void main(void)
{
    mediump float x = OutputSize.x * Texcoord0.x;
    mediump float y = OutputSize.y * Texcoord0.y;
    
    mediump vec4 alphavec = texture2D(AlphaTexture, Texcoord0);
    mediump float alpha = alphavec[3];
    
    mediump vec4 trimap = texture2D(TrimapTexture, Texcoord0);

    gl_FragColor = vec4(0);
    
    if (trimap.r < 0.02 || trimap.r > 0.98)
    {
        gl_FragColor.r = trimap.r;
    }
    else
    {
        mediump vec2 offset;
        
        highp float meanI[3];
        highp float meanA;
        highp float Sigma[9];
        highp float cov_Ip[3];
        highp float a_xy[3];
        
        meanI[0] = meanI[1] = meanI[2] = meanA = 0.0;
        
        for (int j=0; j<9; ++j)
            Sigma[j] = 0.0;
        
        //compute means
        mediump float kernel = 2.0;
        highp float w = OutputSize.x;
        highp float h = OutputSize.y;
        
        highp float lowerx = (0.0 > x-kernel) ? -x : -kernel;
        highp float upperx = (w-1.0 < x+kernel) ? w-1.0-x : kernel;
        highp float lowery = (0.0 > y-kernel) ? -y : -kernel;
        highp float uppery = (h-1.0 < y+kernel) ? h-1.0-y : kernel;
        
        highp float count = 0.0;
        mediump vec4 color;
        for (highp float yy = lowery; yy <= uppery; ++yy)
        {
            for (highp float xx=lowerx; xx<=upperx; ++xx)
            {
                ++count;
                offset[0] = xx / OutputSize.x;
                offset[1] = yy / OutputSize.y;
                
                meanA += texture2D(AlphaTexture, Texcoord0+offset)[3];
                color = texture2D(ImageTexture, Texcoord0+offset);
                meanI[0] += color[0];
                meanI[1] += color[1];
                meanI[2] += color[2];
            }
        }
        
        meanA /= count;
        for (int k = 0; k < 3; ++k)
            meanI[k] /= count;
        
        highp float  invCount = 1.0 / count;
        
        //compute upper part of covariance matrix
        for (highp float yy = lowery; yy <= uppery; ++yy)
        {
            for (highp float xx=lowerx; xx<=upperx; ++xx)
            {
                offset[0] = xx / OutputSize.x;
                offset[1] = yy / OutputSize.y;
                
                color = texture2D(ImageTexture, vec2(Texcoord0.x+offset.x, 1.0-Texcoord0.y-offset.y));
                
                Sigma[0] += (color[0] - meanI[0]) * (color[0] - meanI[0]) * invCount;
                Sigma[1] += (color[0] - meanI[0]) * (color[1] - meanI[1]) * invCount;
                Sigma[2] += (color[0] - meanI[0]) * (color[2] - meanI[2]) * invCount;
                Sigma[4] += (color[1] - meanI[1]) * (color[1] - meanI[1]) * invCount;
                Sigma[5] += (color[1] - meanI[1]) * (color[2] - meanI[2]) * invCount;
                Sigma[8] += (color[2] - meanI[2]) * (color[2] - meanI[2]) * invCount;
            }
        }
        
        Sigma[0] += 0.0001;
        Sigma[4] += 0.0001;
        Sigma[8] += 0.0001;
        
        //compute a_k
        cov_Ip[0] = cov_Ip[1] = cov_Ip[2] = 0.0;
        for (mediump float yy = lowery; yy <= uppery; ++yy)
        {
            for (mediump float xx=lowerx; xx<=upperx; ++xx)
            {
                offset[0] = xx / OutputSize.x;
                offset[1] = yy / OutputSize.y;
                
                color = texture2D(ImageTexture, vec2(Texcoord0.x+offset.x, 1.0-Texcoord0.y-offset.y));
                
                mediump float a = texture2D(AlphaTexture, Texcoord0+offset)[3];
                cov_Ip[0] += (color[0]*a -meanI[0]*meanA) * invCount;
                cov_Ip[1] += (color[1]*a -meanI[1]*meanA) * invCount;
                cov_Ip[2] += (color[2]*a -meanI[2]*meanA) * invCount;
            }
        }
        
        //perform Cholesky decomposition
        Sigma[0] = sqrt(Sigma[0]);
        Sigma[3] = Sigma[1] / Sigma[0];
        Sigma[6] = Sigma[2] / Sigma[0];
        Sigma[4] = sqrt( Sigma[4] - Sigma[3]*Sigma[3]);
        Sigma[7] = (Sigma[5] - Sigma[3]*Sigma[6])/Sigma[4];
        Sigma[8] = Sigma[8] - Sigma[7]*Sigma[7] - Sigma[6]*Sigma[6];
        a_xy[0] = cov_Ip[0] / Sigma[0];
        a_xy[1] = (cov_Ip[1] - Sigma[3]*a_xy[0]) / Sigma[4];
        a_xy[2] = (cov_Ip[2] - Sigma[6]*a_xy[0] - Sigma[7]*a_xy[1]) / Sigma[8];
        a_xy[1] = (a_xy[1] - Sigma[7]*a_xy[2]) / Sigma[4];
        a_xy[0] = (a_xy[0] - Sigma[3]*a_xy[1] - Sigma[6]*a_xy[2]) / Sigma[0];
        
        //gl_FragColor.rgb = alphavec.rgb;
        
        //compute b_k
        highp float b_k = meanA - (a_xy[0]*meanI[0]+a_xy[1]*meanI[1]+a_xy[2]*meanI[2]);
        
        //compute q_i
        color = texture2D(ImageTexture, vec2(Texcoord0.x, 1.0-Texcoord0.y));
        highp float a1 =  a_xy[0]*color[0]+a_xy[1]*color[1]+a_xy[2]*color[2] + b_k;
        a1 = (0.0 > a1) ? 0.0 : a1;
        a1 = (1.0 < a1) ? 1.0 : a1;
        
        gl_FragColor.r = a1;
 
    }//end big else
}
