varying highp vec2      Texcoord0;

uniform sampler2D       ImageTexture;
uniform sampler2D       TrimapTexture;
uniform sampler2D       FgBgTexture;

uniform highp vec2      OutputSize;
uniform int             OutputMode;

//uniform mediump vec3 globalFgSample;
//uniform mediump vec3 globalBgSample;
//uniform lowp vec2 globalFgSampleLoc;
//uniform lowp vec2 globalBgSampleLoc;

//output - fg x,y; bg x,y

void main(void)
{
    const int SAMPLESIZE = 4;
    mediump float SAMPLE_STEPSIZE  = 6.0;
    mediump float SHARING_WINDOW_SIZE = 11.0;
    mediump float PI = 3.14159265;
    
    mediump float x = OutputSize.x * Texcoord0.x;
    mediump float y = OutputSize.y * Texcoord0.y;
    
    mediump vec4 trimap = texture2D(TrimapTexture, Texcoord0);
    
    gl_FragColor = vec4(0);
    
    if (trimap.r < 0.02)
    {
        if (OutputMode == 1)
        {
            gl_FragColor.rgb = texture2D(ImageTexture, vec2(Texcoord0.x, 1.0-Texcoord0.y)).rgb;
        }
        else
        {
            gl_FragColor.r = 0.0;
            gl_FragColor.g = 0.0;
            gl_FragColor.b = 0.0;
        }
        
        gl_FragColor.a = 0.0;
    }
    else if (trimap.r > 0.98)
    {
        if (OutputMode == 0)
        {
            gl_FragColor.rgb = texture2D(ImageTexture, vec2(Texcoord0.x, 1.0-Texcoord0.y)).rgb;
        }
        else
        {
            gl_FragColor.r = 0.0;
            gl_FragColor.g = 0.0;
            gl_FragColor.b = 0.0;
        }
        
        gl_FragColor.a = 1.0;
    }
    else
    {
        mediump vec4 index = texture2D(FgBgTexture, Texcoord0);
        mediump float dir;
        mediump float steps;
        if (OutputMode == 0)
        {
            dir = index.r * 10.0;
            steps = index.g * 255.0;
        }
        else
        {
            dir = index.b * 10.0;
            steps = index.a * 255.0;
        }
        
        if (dir > 9.0)
        {
            if (OutputMode == 0)
            {
                gl_FragColor.rgb = vec3(255.0,255.0,255.0);
            }
            else
            {
                gl_FragColor.rgb = vec3(255.0,255.0,255.0);
            }
        }
        else
        {
            mediump float angle = mod(x,3.0)+mod(y,3.0)*3.0;
            mediump float incangle = 2.0*PI/float(SAMPLESIZE);
            mediump float kfInv9 = 1.0/9.0;
            mediump float currangle = incangle * (angle*kfInv9) + dir*incangle;
            mediump float sina = sin(currangle);
            mediump float cosa = cos(currangle);
            highp float nx = x + steps*SAMPLE_STEPSIZE*cosa;
            highp float ny = y + steps*SAMPLE_STEPSIZE*sina;
            mediump vec2 coord;
            coord.x = nx / float(OutputSize.x);
            coord.y = ny / float(OutputSize.y);
            
            gl_FragColor.rgb = texture2D(ImageTexture, vec2(coord.x, 1.0 - coord.y)).rgb;
            gl_FragColor.a = 1.0;
        }
        
        
    }//end big else

}
