varying highp vec2      Texcoord0;

uniform sampler2D       ImageTexture;
uniform sampler2D       TrimapTexture;

uniform highp vec2      OutputSize;

//uniform mediump vec3 globalFgSample;
//uniform mediump vec3 globalBgSample;
//uniform lowp vec2 globalFgSampleLoc;
//uniform lowp vec2 globalBgSampleLoc;

//output - fg x,y; bg x,y

void main(void)
{
    
    const int SAMPLESIZE  = 4;
    mediump float SAMPLE_STEPSIZE = 6.0;
    mediump float SHARING_WINDOW_SIZE = 11.0;
    mediump float PI = 3.14159265;
    
    mediump float x = OutputSize.x * Texcoord0.x;
    mediump float y = OutputSize.y * Texcoord0.y;
    
    mediump vec4 trimap = texture2D(TrimapTexture, Texcoord0);
    
    if (trimap.r < 0.02)
    {
        gl_FragColor.r = 0.0;
        gl_FragColor.g = 0.0;
        gl_FragColor.b = 0.0;
        gl_FragColor.a = 0.0;
    }
    else if (trimap.r > 0.98)
    {
        gl_FragColor.r = 0.0;
        gl_FragColor.g = 0.0;
        gl_FragColor.b = 0.0;
        gl_FragColor.a = 1.0;
    }
    else
    {
        mediump vec3 fgsamples[SAMPLESIZE];
        mediump vec3 bgsamples[SAMPLESIZE];
        mediump vec2 fgloc[SAMPLESIZE];
        mediump vec2 bgloc[SAMPLESIZE];
        mediump float fgdir[SAMPLESIZE];
        mediump float bgdir[SAMPLESIZE];
        mediump float fgsteps[SAMPLESIZE];
        mediump float bgsteps[SAMPLESIZE];
        
        
        int fgcount = 0;
        int bgcount = 0;
        
        //cast rays
        mediump float angle = mod(x,3.0)+mod(y,3.0)*3.0;
        mediump float incangle = 2.0*PI/float(SAMPLESIZE);
        mediump float kfInv9 = 1.0/9.0;
        mediump float currangle = incangle * (angle*kfInv9);
        mediump vec2 coord;
        mediump float  maxdist = 1500.0;
        highp float w = OutputSize.x;
        highp float h = OutputSize.y;
        
        for (int k = 0; k < SAMPLESIZE; ++k)
        {
            mediump float sina = sin(currangle);
            mediump float cosa = cos(currangle);
            bool fgfound = false;
            bool bgfound = false;
            //step in increments of 1 along line until you go off image
            highp float nx = x + SAMPLE_STEPSIZE * cosa;
            highp float ny = y + SAMPLE_STEPSIZE * sina;
            mediump float step = 1.0;
            for (mediump float s = SAMPLE_STEPSIZE; s < maxdist && nx>=0.0 && nx<w && ny>=0.0 && ny<h && (!fgfound && !bgfound) /*&& s < 1800.0*/; ++step)
            {
                coord.x = nx / w;
                coord.y = ny / h;
                mediump vec4 samp = texture2D(ImageTexture, vec2(coord.x, 1.0-coord.y));
                mediump vec4 tmap = texture2D(TrimapTexture, coord);
                if (!fgfound && tmap.r> 0.98)
                {
                    fgfound = true;
                    fgsamples[fgcount] = samp.rgb;
                    fgloc[fgcount].x = coord.x * w;
                    fgloc[fgcount].y = coord.y * h;
                    fgdir[fgcount] = float(k);
                    fgsteps[fgcount] = step;
                    fgcount++;
                }
                else if (!bgfound && tmap.r< 0.02)
                {
                    bgfound = true;
                    bgsamples[bgcount] = samp.rgb;
                    bgloc[bgcount].x = coord.x * w;
                    bgloc[bgcount].y = coord.y * h;
                    bgdir[bgcount] = float(k);
                    bgsteps[bgcount] = step;
                    bgcount++;
                }
                
                //increment
                s+=SAMPLE_STEPSIZE;
                nx = x + s*cosa;
                ny = y + s*sina;
                
            }
            
            //go on to next angle
            currangle += incangle;
        }
        
        //add global sample
        //coord.x = mod(x, SHARING_WINDOW_SIZE) / SHARING_WINDOW_SIZE;
        //coord.y = mod(y, SHARING_WINDOW_SIZE) / SHARING_WINDOW_SIZE;
        //int found = bgcount;
        //gl_FragColor.a = float(bgcount) / 4.0;
        if (fgcount == 0)
        {
            mediump vec3 globalFgSample = vec3(255.0,255.0,255.0);
            mediump vec2 globalFgSampleLoc = vec2(-100.0,-100.0);
            fgsamples[fgcount] = globalFgSample;
            fgdir[fgcount] = 10.0;
            fgloc[fgcount++] = globalFgSampleLoc;
        }
        if (bgcount == 0)
        {
            mediump vec3 globalBgSample = vec3(255.0,255.0,255.0);
            mediump vec2 globalBgSampleLoc = vec2(-100.0,-100.0);
            bgsamples[bgcount] = globalBgSample;
            bgdir[bgcount] = 10.0;
            bgloc[bgcount++] = globalBgSampleLoc;
        }
        
        //choose best combo
        mediump vec3 best;
        mediump float besta;
        mediump float bestfd;
        mediump float bestfs;
        mediump float bestbd;
        mediump float bestbs;
        
        highp float minVal = 1e10;
        
        for (int f = 0; f < fgcount; ++f)
        {
            for (int b = 0; b < bgcount; ++b)
            {
                mediump float Np = 0.0;
                
                mediump vec3 fcolor = fgsamples[f];
                mediump vec3 bcolor = bgsamples[b];
                mediump vec3 dfb = fcolor-bcolor;
                mediump float val = dot(dfb,dfb);
                
                bool good = (val < 0.0001) ? false : true;
                val = 1.0 / val;
                mediump float a;
                mediump float ybound = min(y+1.0, h-1.0);
                mediump float xbound = min(x+1.0, w-1.0);
                for (highp float yy = max(0.0,y-1.0); yy <= ybound; ++yy)
                    for (highp float xx = max(0.0,x-1.0); xx <= xbound; ++xx)
                    {
                        coord.x = xx / w;
                        coord.y = yy / h;
                        mediump vec3 tcolor = texture2D(ImageTexture, vec2(coord.x, 1.0-coord.y)).rgb;
                        if (good)
                        {
                            mediump vec3 d = (bcolor-tcolor)*(fcolor-bcolor);
                            a =  - ( d.r + d.g + d.b ) / val;
                            a = max(0.0, min(1.0,a));
                        }
                        else a = 0.5;
                        
                        mediump vec3 d =  tcolor - (a*fcolor+(1.0-a)*bcolor);
                        Np += dot(d, d);
                        
                    }
                
                mediump vec2 temp = vec2(x,y) - fgloc[f];
                temp = vec2(x,y) - bgloc[b];
                
                highp float energy = Np;// * Dpf*Dpf*Dpf * Dpb*Dpb*Dpb*Dpb;
                
                if (energy < minVal)
                {
                    minVal = energy;
                    bestfd = fgdir[f];
                    bestfs = fgsteps[f];
                    bestbd = bgdir[b];
                    bestbs = bgsteps[b];
                    
                    /*coord.x = x / w;
                     coord.y = y / h;
                     mediump vec3 tcolor = texture2D(ImageTexture, coord).rgb;
                     mediump vec3 d = (bcolor-tcolor)*(fcolor-bcolor);
                     a =  - ( d.r + d.g + d.b ) / val;
                     besta = max(0.0, min(1.0,a));
                     */
                    
                }
            }
        }
        
        gl_FragColor.r = bestfd / 10.0;
        gl_FragColor.g = bestfs / 255.0;
        gl_FragColor.b = bestbd / 10.0;
        gl_FragColor.a = bestbs / 255.0;
        
        //gl_FragColor.r = 1.0;//bestfd / 4.0;
        //gl_FragColor.g = 0.0;//bestfs / 20.0;
        //gl_FragColor.b = 0.0;
        //gl_FragColor.a = 1.0;
        
        //gl_FragColor.a = besta;
        //gl_FragColor.a = float(found) / 4.0;
        
    }//end big else

}
