#version 300 es

in highp vec2 Texcoord0;

uniform highp vec3  Center0;
uniform highp vec3  Center1;

uniform highp float RadiusCircle;

out highp vec4 g_FragColor;

highp float line_width = 0.02;
highp float PI = 3.14159;

highp vec4 RenderIntersection(highp vec2 coord, highp float distance0, highp float distance1)
{
    // line interval
    highp float interval = 0.1;
    
    highp vec4 ret = vec4(0.7, 0.95, 1.0, 1.0);
    
    highp float lineAngle = -PI * 0.25;
    
    // lines
    highp float texcoord_x = coord.x*cos(lineAngle)-coord.y*sin(lineAngle) + 1.0;
    
    if( fract(texcoord_x / interval)*interval < line_width)
    {
        return ret;
    }

    // borders
    if( (RadiusCircle - distance0) < line_width && (RadiusCircle - distance0) > 0.0)
    {
        return ret;
    }
    
    return vec4(0);
}

highp vec4 RenderCircle0(highp vec2 coord)
{
    return vec4(0.14, 0.78, 0.99, 1.0);
}

highp vec4 RenderCircle1(highp vec2 coord, highp float distance1)
{
    highp float interval = PI * 0.1;
    
    if( (RadiusCircle - distance1) < line_width && (RadiusCircle - distance1) > 0.0 )
    {
        // offset by one period so that step is always > 0
        int step = int( (atan((coord.y-Center1.y)/(coord.x-Center1.x))+PI*0.5) / interval);
        
        highp vec4 ret = vec4(0.14, 0.78, 0.99, 1.0);
        
        if( step % 2 == 0 )
        {
            return ret;
        }
    }
    
    return vec4(0);
}

void main()
{
    g_FragColor = vec4(0);
    
    // transform texture coord to effect coord (0, 1) -> (-1, 1)
    highp vec2 coord = vec2(Texcoord0.x*2.0-1.0, Texcoord0.y*2.0-1.0);
    
    highp float distance0 = length(coord - vec2(Center0));
    highp float distance1 = length(coord - vec2(Center1));
    
    // not in both
    if( distance0 > RadiusCircle && distance1 > RadiusCircle )
    {
        g_FragColor = vec4(0);
    }
    // in both
    else if(distance0 <= RadiusCircle && distance1 <= RadiusCircle)
    {
        g_FragColor = RenderIntersection(coord, distance0, distance1);
    }
    // only in circle 0
    else if(distance0 <= RadiusCircle)
    {
        g_FragColor = RenderCircle0(coord);
    }
    // only in circle 1
    else if(distance1 <= RadiusCircle)
    {
        g_FragColor = RenderCircle1(coord, distance1);
    }
}

