#version 300 es

in highp vec2           Texcoord0;

uniform sampler2D       ImageTexture;
uniform sampler2D       TrimapTexture;
uniform sampler2D       ForegroundTexture;
uniform sampler2D       BackgroundTexture;

uniform highp vec2      OutputSize;

out highp vec4          g_FragColor;

//uniform mediump vec3 globalFgSample;
//uniform mediump vec3 globalBgSample;
//uniform lowp vec2 globalFgSampleLoc;
//uniform lowp vec2 globalBgSampleLoc;

//output - fg x,y; bg x,y

void main(void)
{
    mediump float x = OutputSize.x * Texcoord0.x;
    mediump float y = OutputSize.y * Texcoord0.y;
    
    mediump vec4 trimap = texture(TrimapTexture, Texcoord0);
    
    if (trimap.r < 0.02 || trimap.r > 0.98)
    {
        g_FragColor.rgb = texture(ImageTexture, vec2(Texcoord0.x, 1.0-Texcoord0.y)).rgb;
        
        g_FragColor.a = trimap.r;
    }
    else
    {
        //start sharing
        mediump float kernel = 2.0;
        highp float w = OutputSize.x;
        highp float h = OutputSize.y;
        highp float lowerx = (0.0 > x-kernel) ? -x : -kernel;
        highp float upperx = (w-1.0 < x+kernel) ? w-1.0-x : kernel;
        highp float lowery = (0.0 > y-kernel) ? -y : -kernel;
        highp float uppery = (h-1.0 < y+kernel) ? h-1.0-y : kernel;
        
        mediump vec4 bestF[3];
        mediump vec4 bestB[3];
        highp float score[3];
        score[0] = score[1] = score[2] = 1e10;
        
        mediump vec4 i = texture(ImageTexture, vec2(Texcoord0.x, 1.0-Texcoord0.y));
        
        mediump vec2 offset;
        for (highp float yy = lowery; yy <= uppery; ++yy)
            for (highp float xx=lowerx; xx<=upperx; ++xx)
            {
                offset[0] = xx / OutputSize.x;
                offset[1] = yy / OutputSize.y;
                
                mediump vec4 f = texture(ForegroundTexture, Texcoord0+offset);
                mediump vec4 b = texture(BackgroundTexture, Texcoord0+offset);
                
                //compute alpha
                mediump float a;
                mediump vec4 dbf = (b-f)*(b-f);
                mediump float val = dbf.r+dbf.g+dbf.b;
                if (val < 0.0001)
                    a = 0.5;
                else
                {
                    mediump vec4 d = (b-i)*(f-b);
                    a =  - ( d.r + d.g + d.b ) / val;
                    //a = (0.0 > a) ? 0.0 : a;
                    //a = (1.0 < a) ? 1.0 : a;
                    a = max(0.0, min(1.0,a));
                }
                
                //compute squared matting error
                mediump vec4 d =  i - (a*f+(1.0-a)*b);
                d = d*d;
                highp float dist = d.r+d.g+d.b;
                
                highp float alphabias = 50.0*(0.5 - abs(a-0.5));
                dist *= 1.0+alphabias*alphabias;
                
                //compare
                if (dist < score[0])
                {
                    score[2] = score[1];
                    score[1] = score[0];
                    score[0] = dist;
                    bestF[2] = bestF[1];
                    bestF[1] = bestF[0];
                    bestF[0] = f;
                    bestB[2] = bestB[1];
                    bestB[1] = bestB[0];
                    bestB[0] = b;
                }
                else if (dist < score[1])
                {
                    score[2] = score[1];
                    score[1] = dist;
                    bestF[2] = bestF[1];
                    bestF[1] = f;
                    bestB[2] = bestB[1];
                    bestB[1] = b;
                    
                }
                else if (dist < score[2])
                {
                    score[2] = dist;
                    bestF[2] = f;
                    bestB[2] = b;
                }
                
            }
        
        mediump vec4 fout = (bestF[0] + bestF[1] + bestF[2]) / 3.0;
        mediump vec4 bout = (bestB[0] + bestB[1] + bestB[2]) / 3.0;
        
        //check if it should be pure f or b
        mediump vec4 d = (fout-i)*(fout-i);
        if (d.r + d.g + d.b < 0.0004)
            fout = i;
        d = (bout-i)*(bout-i);
        if (d.r + d.g + d.b < 0.0004)
            bout = i;
        
        //compute final alpha
        mediump float a;
        mediump vec4 dbf = (bout-fout)*(bout-fout);
        mediump float val = dbf.r+dbf.g+dbf.b;
        if (val < 0.0001)
            a = 0.5;
        else
        {
            mediump vec4 d = (bout-i)*(fout-bout);
            a =  - ( d.r + d.g + d.b ) / val;
            //a = (0.0 > a) ? 0.0 : a;
            //a = (1.0 < a) ? 1.0 : a;
            a = max(0.0, min(1.0,a));
        }
        
        g_FragColor.a = a;
        
        g_FragColor.rgb = fout.rgb;
    }//end big else
}
