#version 300 es
// ui_2d_element_render.fsh

in highp vec2       texCoordOut;
in highp vec2       imgTexCoordOut;
in highp vec2       oldTexCoordOut;
in highp vec2       maskTexCoordOut;

uniform highp float alpha;

uniform highp vec2  clipLeftTop;
uniform highp vec2  clipRightBottom;

uniform int         needStencil;
uniform sampler2D   stencilTex;
uniform highp vec2  stencilTexSize;
uniform int         renderIndexFirst;
uniform int         renderIndexLast;

// loading effect parameter.
uniform highp vec2  Center0;
uniform highp vec2  Center1;

uniform highp float RadiusCircle;
uniform highp float LineWidth;

highp float PI = 3.14159;

out highp vec4 g_FragColor;

highp vec4 RenderIntersection(highp vec2 coord, highp float distance0, highp float distance1)
{
    // line interval
    highp float interval = RadiusCircle / 5.0;
    
    highp vec4 ret = vec4(0.7, 0.95, 1.0, 1.0);
    
    highp float lineAngle = -PI * 0.25;
    
    // lines
    highp float texcoord_x = coord.x*cos(lineAngle)-coord.y*sin(lineAngle) + 1.0;
    
    if( fract(texcoord_x / interval)*interval < LineWidth)
    {
        return ret;
    }
    
    // borders
    if( (RadiusCircle - distance0) < LineWidth && (RadiusCircle - distance0) > 0.0)
    {
        return ret;
    }
    
    return vec4(0);
}

highp vec4 RenderCircle0(highp vec2 coord)
{
    return vec4(0.14, 0.78, 0.99, 1.0);
}

highp vec4 RenderCircle1(highp vec2 coord, highp float distance1)
{
    highp float interval = PI * 0.1;
    
    if( (RadiusCircle - distance1) < LineWidth && (RadiusCircle - distance1) > 0.0 )
    {
        // offset by one period so that step is always > 0
        int step = int( (atan((coord.y-Center1.y)/(coord.x-Center1.x))+PI*0.5) / interval);
        
        highp vec4 ret = vec4(0.14, 0.78, 0.99, 1.0);
        
        if( step % 2 == 0 )
        {
            return ret;
        }
    }
    
    return vec4(0);
}

void main()
{
    g_FragColor = vec4(0);
    
    highp vec2 coord = vec2(gl_FragCoord.x, gl_FragCoord.y);
    
    highp float distance0 = length(coord - Center0);
    highp float distance1 = length(coord - Center1);
    
    // not in both
    if( distance0 > RadiusCircle && distance1 > RadiusCircle )
    {
        g_FragColor = vec4(0);
    }
    // in both
    else if(distance0 <= RadiusCircle && distance1 <= RadiusCircle)
    {
        g_FragColor = RenderIntersection(coord, distance0, distance1);
    }
    // only in circle 0
    else if(distance0 <= RadiusCircle)
    {
        g_FragColor = RenderCircle0(coord);
    }
    // only in circle 1
    else if(distance1 <= RadiusCircle)
    {
        g_FragColor = RenderCircle1(coord, distance1);
    }
    
    g_FragColor.a = alpha * g_FragColor.a;
    
    if (needStencil == 1)
    {
        highp vec2 coord;
        coord.x = gl_FragCoord.x / stencilTexSize.x;
        coord.y = gl_FragCoord.y / stencilTexSize.y;
        highp vec4 c = texture(stencilTex, coord, 0.0);
        
        int index = int(c.r * 100.0 + 0.5) + int(c.g * 100.0 + 0.5) * 100 + int(c.b * 100.0 + 0.5) * 10000;
        if (index < renderIndexFirst || index > renderIndexLast)
        {
            g_FragColor.a = 0.0;
        }
    }
    else
    {
        g_FragColor.a = g_FragColor.a * float(texCoordOut.x >= clipLeftTop.x && texCoordOut.x <= clipRightBottom.x && texCoordOut.y >= clipRightBottom.y && texCoordOut.y <= clipLeftTop.y);
    }
    
    return;
}