varying highp vec4      Position0;
varying highp vec2      Texcoord0;

uniform highp vec2      FlowOffset;
uniform highp float     Scale;

uniform highp mat4      matNormal;
uniform sampler2D       BumpMap;

highp vec3 GetNormalDir()
{
    highp vec2 offset = FlowOffset;
    highp vec2 scaled_texcoord = Texcoord0*Scale;
    
    highp vec2 bumpTexcoord0 = vec2(fract(scaled_texcoord.x+offset.x), fract(scaled_texcoord.y));
    highp vec2 bumpTexcoord1 = vec2(fract(scaled_texcoord.x), fract(scaled_texcoord.y+offset.y));
    
    //    highp vec2 bumpTexcoord2 = vec2(fract(scaled_texcoord.x-offset.x), fract(scaled_texcoord.y));
    //    highp vec2 bumpTexcoord3 = vec2(fract(scaled_texcoord.x), fract(scaled_texcoord.y-offset.y));
    
    // WARNING: bump map must be aligned with xy plane initially
    highp vec4 bumpDir0 = texture2D(BumpMap, bumpTexcoord0);
    highp vec4 bumpDir1 = texture2D(BumpMap, bumpTexcoord1);
    //    highp vec4 bumpDir2 = texture(BumpMap, bumpTexcoord2);
    //    highp vec4 bumpDir3 = texture(BumpMap, bumpTexcoord3);
    
    bumpDir0 = bumpDir0-vec4(0.5, 0.5, 0.5, 1);
    bumpDir1 = bumpDir1-vec4(0.5, 0.5, 0.5, 1);
    //    bumpDir2 = bumpDir2-vec4(0.5);
    //    bumpDir3 = bumpDir3-vec4(0.5);
    
    // TODO: water normal need to be passed in
    // normal scale is hard coded
    highp vec3 normalDir = vec3(bumpDir0+bumpDir1);
    normalDir = vec3(matNormal*vec4(normalDir, 0));
    
    highp vec3 bumpDir = normalize(normalDir);
    return bumpDir;
}

highp vec3 CompressNormalDir(highp vec3 dir)
{
    return (dir+vec3(1.0))*0.5;
}

void main()
{
    highp float z = Position0.z/Position0.w;
    z = (z+1.0)*0.5;
    
    gl_FragColor = vec4(CompressNormalDir(GetNormalDir()), z);
}
