#version 300 es

in highp vec4 Position0;
in highp vec3 Normal0;
in highp vec2 Texcoord0;

out highp vec4 g_FragColor;

uniform bool        HasTexture;
uniform sampler2D   Texture0;

uniform highp vec4  EyePos;
uniform highp vec4  LightSourcePos;

uniform highp vec4  K_a;
uniform highp vec4  K_s;
uniform highp vec4  K_d;
uniform highp float Shininess;

highp float specularComponent(highp vec4 lightSourcePos, highp vec4 eyePos, highp vec3 normal, highp vec4 pos, highp float shininess)
{
    // normal must be normalized
    highp vec3 _normal = normal;//normalize(normal);
    
    highp vec4 incident = pos-lightSourcePos;
    highp vec4 reflection = normalize(reflect(incident, vec4(_normal, 0)));
    
    highp vec4 lookat = normalize(eyePos-pos);
    
    highp float _s = max(0.0, dot(lookat, reflection));
    _s = pow(_s, shininess);
    
    return _s;
}

highp float diffuseComponent(highp vec4 lightSourcePos, highp vec3 normal, highp vec4 pos)
{
    highp float _d = dot(normalize(lightSourcePos-pos), vec4(normal,0));
    
    return max(0.0, _d);
}

void main()
{
    highp vec4 lightSourcePos = LightSourcePos;
    highp vec4 eyePos = EyePos;
    
    highp vec4 ambient = vec4(0.0);

    if(HasTexture)
    {
        ambient = texture(Texture0, Texcoord0);
    }
    else
    {
        ambient = vec4(1,0,0,1);
    }

    highp vec4 ambient_comp = ambient;
    
    highp vec4 diffuse_comp = ambient * diffuseComponent(lightSourcePos, Normal0, Position0);//vec4(Normal0,1);

    highp vec4 specular_comp = ambient * specularComponent(lightSourcePos, eyePos, Normal0, Position0, Shininess);
    
    g_FragColor = K_a*ambient_comp + K_d*diffuse_comp+K_s*specular_comp;
    
    g_FragColor.a = 1.0;
}
