varying highp vec2       texCoordOut;
varying highp vec2       imgTexCoordOut;
varying highp vec2       oldTexCoordOut;
varying highp vec2       maskTexCoordOut;
varying highp float outObjIndex;
varying highp vec4  bgColor;
varying highp vec4  colorInfo1;
varying highp vec4  colorInfo2;
varying highp vec4  clipRect;

// texture used
uniform sampler2D   texture0;
uniform sampler2D   texture1;
uniform sampler2D   texture2;
uniform sampler2D   texture3;
uniform sampler2D   texture4;
uniform sampler2D   texture5;

highp vec4 fetchTexture(int idx, highp vec2 coord, highp float mipmapLevel)
{
    if(idx==0)
    {
        return texture2D(texture0, coord, mipmapLevel);
    }
    else if(idx==1)
    {
        return texture2D(texture1, coord, mipmapLevel);
    }
    else if(idx==2)
    {
        return texture2D(texture2, coord, mipmapLevel);
    }
    else if(idx==3)
    {
        return texture2D(texture3, coord, mipmapLevel);
    }
    else if(idx==4)
    {
        return texture2D(texture4, coord, mipmapLevel);
    }
    else if(idx==5)
    {
        return texture2D(texture5, coord, mipmapLevel);
    }
    
    return vec4(0.0, 0.0, 0.0, 0.0);
}

void main()
{
    highp float colorMode = colorInfo1.r;
    highp float blendingFactor = colorInfo1.g;
    highp float mipmapLevel = colorInfo1.b;
    highp float imgTexIndex = colorInfo1.a;
    highp float oldImgTexIndex = colorInfo2.r;
    highp float hasMask = colorInfo2.g;
    highp float maskTexIndex = colorInfo2.b;
    highp float alpha = colorInfo2.a;
    
    // solid color
    if (colorMode < 1.5)
    {
        gl_FragColor = bgColor;
    }
    // texture used.
    else if (colorMode < 2.5)
    {
        highp vec4 color = fetchTexture(int(imgTexIndex), imgTexCoordOut, mipmapLevel);
        if (blendingFactor >= 1.0)
        {
            gl_FragColor = color;
        }
        else
        {
            highp vec4 oldColor = fetchTexture(int(oldImgTexIndex), oldTexCoordOut, mipmapLevel);
            gl_FragColor = color * float(color.a != 0.0) * blendingFactor + oldColor * float(oldColor.a != 0.0) * (1.0 - blendingFactor);
        }
    }

    if (hasMask == 1.0)
    {
        gl_FragColor.a = fetchTexture(int(maskTexIndex), maskTexCoordOut, 0.0).a * gl_FragColor.a;
    }

    gl_FragColor.a = gl_FragColor.a * float(texCoordOut.x >= clipRect.x && texCoordOut.x <= clipRect.z && texCoordOut.y >= clipRect.w && texCoordOut.y <= clipRect.y);
    
    gl_FragColor.a = alpha * gl_FragColor.a;
}