attribute vec3     Position;
attribute vec3     Normal;
attribute vec2     Texcoord;
attribute float    MeshIndex;

uniform mat4        matVP;
uniform highp float renderInfoBufferWidth;
uniform highp float renderInfoBufferHeight;
uniform sampler2D   renderInfoBuffer;

varying highp vec2  texCoordOut;
varying highp vec2  imgTexCoordOut;
varying highp vec2  oldTexCoordOut;
varying highp vec2  maskTexCoordOut;

varying highp float outObjIndex;
varying highp vec4  bgColor;
varying highp vec4  colorInfo1;
varying highp vec4  colorInfo2;
varying highp vec4  clipRect;

// functions to fetch value from info buffer

highp vec4 fetchValue(int x)
{   
	highp float bufferStrideFactor = (1.0 / renderInfoBufferWidth);
	highp float infoRowIdx = (MeshIndex + 0.5) / renderInfoBufferHeight;
	highp float col = (float(x) + 0.5) * bufferStrideFactor;
    return texture2D(renderInfoBuffer, vec2(col, infoRowIdx));
}

mat4 fetchMatrix(int x)
{
    highp vec4 col0 = fetchValue(x);
    highp vec4 col1 = fetchValue(x + 1);
    highp vec4 col2 = fetchValue(x + 2);
    highp vec4 col3 = fetchValue(x + 3);
    
    highp mat4 mat = mat4(col0,
                              col1,
                              col2,
                              col3);
    
    return mat;
}

void main()
{
    mat4 transMat = fetchMatrix(0);
    mat4 imgTexMat = fetchMatrix(4);
    mat4 oldImgTexMat = fetchMatrix(8);
    mat4 maskTexMat = fetchMatrix(12);
    
    gl_Position = matVP * transMat * vec4(Position, 1);
    
    texCoordOut = Texcoord;
    
    highp vec4 texCoordV4;
    texCoordV4.xy = Texcoord.xy;
    texCoordV4.z = 0.0;
    texCoordV4.w = 1.0;
    
    imgTexCoordOut.xy = (imgTexMat * texCoordV4).xy;
    oldTexCoordOut.xy = (oldImgTexMat * texCoordV4).xy;
    maskTexCoordOut.xy = (maskTexMat * texCoordV4).xy;
    
    outObjIndex = MeshIndex;
    
    bgColor = fetchValue(16);
    
    colorInfo1 = fetchValue(17);
    
    colorInfo2 = fetchValue(18);
    
    clipRect = fetchValue(19);
    
    //needClip = float(clipRect.x > 0.0 || clipRect.y < 1.0 || clipRect.z < 1.0 || clipRect.w > 0.0);
}
