
// ui_2d_element_render.fsh

varying highp vec2  texCoordOut;
varying highp vec2  imgTexCoordOut;
varying highp vec2  oldTexCoordOut;
varying highp vec2  maskTexCoordOut;

uniform highp float alpha;

uniform sampler2D   texture0;
uniform sampler2D   textureOld;
uniform int         channelCount0;
uniform int         channelCountOld;

uniform highp float blendingFactor;

uniform highp vec2  clipLeftTop;
uniform highp vec2  clipRightBottom;

uniform int         needStencil;
uniform sampler2D   stencilTex;
uniform highp vec2  stencilTexSize;
uniform int         renderIndexFirst;
uniform int         renderIndexLast;

uniform highp float mipmapLevel;

highp vec4 textureSample(sampler2D tex, highp vec2 coord, int channelCount)
{
    highp vec4 color;
    if (channelCount == 4)
    {
        color = texture2D(tex, coord, mipmapLevel);
    }
    else if (channelCount == 1)
    {
        color.r = texture2D(tex, coord, mipmapLevel).r;
        color.g = 0.0;
        color.b = 0.0;
        color.a = 1.0;
    }
    else if (channelCount == 3)
    {
        color.rgb = texture2D(tex, coord, mipmapLevel).rgb;
        color.a = 1.0;
    }
    else if (channelCount == 2)
    {
        color.rg = texture2D(tex, coord, mipmapLevel).rg;
        color.b = 0.0;
        color.a = 1.0;
    }
    
    return color;
}

void main()
{
    highp vec4 color = textureSample(texture0, imgTexCoordOut, channelCount0);

    highp vec4 oldColor = textureSample(textureOld, oldTexCoordOut, channelCountOld);
    
    gl_FragColor = color * float(color.a != 0.0) * blendingFactor + oldColor * float(oldColor.a != 0.0) * (1.0 - blendingFactor);
    
    gl_FragColor.a = alpha * gl_FragColor.a;
    
    if (needStencil == 1)
    {
        highp vec2 coord;
        coord.x = gl_FragCoord.x / stencilTexSize.x;
        coord.y = gl_FragCoord.y / stencilTexSize.y;
        highp vec4 c = texture2D(stencilTex, coord, 0.0);
        
        int index = int(c.r * 100.0 + 0.5) + int(c.g * 100.0 + 0.5) * 100 + int(c.b * 100.0 + 0.5) * 10000;
        if (index < renderIndexFirst || index > renderIndexLast)
        {
            gl_FragColor.a = 0.0;
        }
    }
    else
    {
        gl_FragColor.a = gl_FragColor.a * float(texCoordOut.x >= clipLeftTop.x && texCoordOut.x <= clipRightBottom.x && texCoordOut.y >= clipRightBottom.y && texCoordOut.y <= clipLeftTop.y);
    }

    return;
}