
// ui_2d_element_render.fsh
varying highp vec2  texCoordOut;
varying highp vec2  imgTexCoordOut;
varying highp vec2  oldTexCoordOut;
varying highp vec2  maskTexCoordOut;

uniform highp float alpha;

uniform sampler2D   texture0;
uniform sampler2D   textureOld;

uniform highp float blendingFactor;

uniform highp vec2  clipLeftTop;
uniform highp vec2  clipRightBottom;

uniform int         needStencil;
uniform sampler2D   stencilTex;
uniform highp vec2  stencilTexSize;
uniform int         renderIndexFirst;
uniform int         renderIndexLast;

uniform highp float mipmapLevel;

uniform highp float checkerSize;
uniform highp vec2  checkerStartPos;

void main()
{
    highp vec4 color = texture2D(texture0, imgTexCoordOut, mipmapLevel);
    highp vec4 oldColor = texture2D(textureOld, oldTexCoordOut, mipmapLevel);
    
    gl_FragColor = color * float(color.a != 0.0) * blendingFactor + oldColor * float(oldColor.a != 0.0) * (1.0 - blendingFactor);
    
    // checker compute
    highp float size = checkerSize * 2.0;
    highp float x = mod(gl_FragCoord[0] - checkerStartPos.x, size);
    highp float y = mod(gl_FragCoord[1] - checkerStartPos.y, size);
    
    x = x - checkerSize;
    y = y - checkerSize;
    
    highp float v = (x*y > 0.0) ? 0.91765 : 1.0;
    
    highp vec4 checkerClr = vec4(v,v,v,1.0);
    
    gl_FragColor = gl_FragColor.a * gl_FragColor + checkerClr * (1.0 - gl_FragColor.a);
    gl_FragColor.a = 1.0;
    
    gl_FragColor.a = alpha * gl_FragColor.a;
    
    if (needStencil == 1)
    {
        highp vec2 coord;
        coord.x = gl_FragCoord.x / stencilTexSize.x;
        coord.y = gl_FragCoord.y / stencilTexSize.y;
        highp vec4 c = texture2D(stencilTex, coord, 0.0);
        
        int index = int(c.r * 100.0 + 0.5) + int(c.g * 100.0 + 0.5) * 100 + int(c.b * 100.0 + 0.5) * 10000;
        if (index < renderIndexFirst || index > renderIndexLast)
        {
            gl_FragColor.a = 0.0;
        }
    }
    else
    {
        gl_FragColor.a = gl_FragColor.a * float(texCoordOut.x >= clipLeftTop.x && texCoordOut.x <= clipRightBottom.x && texCoordOut.y >= clipRightBottom.y && texCoordOut.y <= clipLeftTop.y);
    }
}