#version 300 es

// ui_2d_element_render.fsh

in highp vec2       texCoordOut;
in highp vec2       imgTexCoordOut;
in highp vec2       oldTexCoordOut;
in highp vec2       maskTexCoordOut;
flat in highp vec4  bgColor;
flat in highp vec4  colorInfo1;
flat in highp vec4  colorInfo2;
flat in highp vec4  clipRect;
//flat in highp float needClip;

// texture used
uniform sampler2D   texture0;
uniform sampler2D   texture1;
uniform sampler2D   texture2;
uniform sampler2D   texture3;
uniform sampler2D   texture4;
uniform sampler2D   texture5;

out highp vec4      g_FragColor;

highp vec4 fetchTexture(int idx, in highp vec2 coord, in highp float mipmapLevel)
{
    switch (idx)
    {
        case 0:
            return texture(texture0, coord, mipmapLevel);
            break;
            
        case 1:
            return texture(texture1, coord, mipmapLevel);
            break;
            
        case 2:
            return texture(texture2, coord, mipmapLevel);
            break;
            
        case 3:
            return texture(texture3, coord, mipmapLevel);
            break;
            
        case 4:
            return texture(texture4, coord, mipmapLevel);
            break;
            
        case 5:
            return texture(texture5, coord, mipmapLevel);
            break;
            
        default:
            break;
    }
    
    return vec4(0.0, 0.0, 0.0, 1.0);
}

void main()
{
    highp float colorMode = colorInfo1.r;
    highp float blendingFactor = colorInfo1.g;
    highp float mipmapLevel = colorInfo1.b;
    highp float imgTexIndex = colorInfo1.a;
    highp float oldImgTexIndex = colorInfo2.r;
    highp float hasMask = colorInfo2.g;
    highp float maskTexIndex = colorInfo2.b;
    highp float alpha = colorInfo2.a;
    
    // solid color
    if (colorMode < 1.5)
    {
        g_FragColor = bgColor;
    }
    // texture used.
    else if (colorMode < 2.5)
    {
        highp vec4 color = fetchTexture(int(imgTexIndex), imgTexCoordOut, mipmapLevel);
        if (blendingFactor >= 1.0)
        {
            g_FragColor = color;
        }
        else
        {
            highp vec4 oldColor = fetchTexture(int(oldImgTexIndex), oldTexCoordOut, mipmapLevel);
            g_FragColor = color * float(color.a != 0.0) * blendingFactor + oldColor * float(oldColor.a != 0.0) * (1.0 - blendingFactor);
        }
    }
    
    g_FragColor.a = fetchTexture(int(maskTexIndex), maskTexCoordOut, 0.0).a * g_FragColor.a;
    
    g_FragColor.a = g_FragColor.a * float(texCoordOut.x >= clipRect.x && texCoordOut.x <= clipRect.z && texCoordOut.y >= clipRect.w && texCoordOut.y <= clipRect.y);
    
    g_FragColor.a = alpha * g_FragColor.a;
}