
#version 300 es
// ui_2d_element_render.vsh

in vec3     Position;
in vec3     Normal;
in vec2     Texcoord;
in float    MeshIndex;

uniform mat4        matVP;
uniform highp float renderInfoBufferWidth;
uniform highp float renderInfoBufferHeight;
uniform sampler2D   renderInfoBuffer;

out highp vec2          texCoordOut;
out highp vec2          imgTexCoordOut;
out highp vec2          oldTexCoordOut;
out highp vec2          maskTexCoordOut;
flat out highp vec4     bgColor;
flat out highp vec4     colorInfo1;
flat out highp vec4     colorInfo2;
flat out highp vec4     clipRect;

// functions to fetch value from info buffer
highp float bufferStrideFactor = 1.0 / renderInfoBufferWidth;
highp float infoRowIdx = (MeshIndex + 0.5) / renderInfoBufferHeight;
highp vec4 fetchValue(int x)
{    highp float col = (float(x) + 0.5) * bufferStrideFactor;
    return texture(renderInfoBuffer, vec2(col, infoRowIdx));
}

mat4x4 fetchMatrix(int x)
{
    highp vec4 col0 = fetchValue(x);
    highp vec4 col1 = fetchValue(x + 1);
    highp vec4 col2 = fetchValue(x + 2);
    highp vec4 col3 = fetchValue(x + 3);
    
    highp mat4x4 mat = mat4x4(col0,
                              col1,
                              col2,
                              col3);
    
    return mat;
}

void main()
{
    mat4x4 transMat = fetchMatrix(0);
    mat4x4 imgTexMat = fetchMatrix(4);
    mat4x4 oldImgTexMat = fetchMatrix(8);
    mat4x4 maskTexMat = fetchMatrix(12);
    
    gl_Position = matVP * transMat * vec4(Position, 1);
    
    texCoordOut = Texcoord;
    
    highp vec4 texCoordV4;
    texCoordV4.xy = Texcoord.xy;
    texCoordV4.z = 0.0;
    texCoordV4.w = 1.0;
    
    imgTexCoordOut.xy = (imgTexMat * texCoordV4).xy;
    oldTexCoordOut.xy = (oldImgTexMat * texCoordV4).xy;
    maskTexCoordOut.xy = (maskTexMat * texCoordV4).xy;
    
    bgColor = fetchValue(16);
    
    colorInfo1 = fetchValue(17);
    
    colorInfo2 = fetchValue(18);
    
    clipRect = fetchValue(19);
    
    //needClip = float(clipRect.x > 0.0 || clipRect.y < 1.0 || clipRect.z < 1.0 || clipRect.w > 0.0);
}
