
// ui_2d_element_render.fsh

varying highp vec4 colorVarying;
varying highp vec2 TextCoordOut;

uniform highp float  alpha;

varying highp vec2   maskTexCoordOut;

uniform highp vec2 clipLeftTop;
uniform highp vec2 clipRightBottom;

uniform highp vec4   gradientColorBegin;
uniform highp vec4   gradientColorEnd;
uniform highp vec2   gradientDir;
uniform highp float  gradientFactor;

highp float _coefficientsX[4];
highp float _coefficientsY[4];

highp float evaluateAtParameterWithCoefficients(mediump float t, mediump float coefficients[4])
{
    return coefficients[0] + t * coefficients[1] + t * t * coefficients[2] + t * t * t * coefficients[3];
}

highp float evaluateDerivationAtParameterWithCoefficients(mediump float t, mediump float coefficients[4])
{
    return coefficients[1] + 2.0 * t * coefficients[2] + 3.0 * t * t * coefficients[3];
}

highp float calcParameterViaNewtonRaphsonUsingXAndCoefficientsForX(mediump float x, mediump float coefficientsX[4])
{
    // start with X being the correct value
    highp float t = x;
    
    // iterate several times
    highp float x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    highp float d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    highp float dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    
    x2 = evaluateAtParameterWithCoefficients(t, coefficientsX) - x;
    d = evaluateDerivationAtParameterWithCoefficients(t, coefficientsX);
    dt = x2 / d;
    t = t - dt;
    return t;
}

highp float calcParameterUsingXAndCoefficientsForX (mediump float x, mediump float coefficientsX[4])
{
    // use Newton-Raphson.
    highp float t = calcParameterViaNewtonRaphsonUsingXAndCoefficientsForX(x, coefficientsX);
    return t;
}

void initBezier(mediump float x1, mediump float y1, mediump float x2, mediump float y2)
{
    highp float x0 = 0.0;
    highp float y0 = 0.0;
    highp float x3 = 1.0;
    highp float y3 = 1.0;
    _coefficientsX[0] = x0; // t^0
    _coefficientsX[1] = (-3.0*x0 + 3.0*x1); // t^1
    _coefficientsX[2] = (3.0*x0 - 6.0*x1 + 3.0*x2);  // t^2
    _coefficientsX[3] = (-x0 + 3.0*x1 - 3.0*x2 + x3); // t^3
    
    _coefficientsY[0] = y0; // t^0
    _coefficientsY[1] = (-3.0*y0 + 3.0*y1); // t^1
    _coefficientsY[2] = (3.0*y0 - 6.0*y1 + 3.0*y2);  // t^2
    _coefficientsY[3] = (-y0 + 3.0*y1 - 3.0*y2 + y3); // t^3
}

highp float valueBezier(mediump float x)
{
    highp float t = calcParameterUsingXAndCoefficientsForX(x, _coefficientsX);
    highp float y = evaluateAtParameterWithCoefficients(t, _coefficientsY);
    return y;
}

bool insideTriangle(highp vec2 coord, highp vec2 a, highp vec2 b, highp vec2 c)
{
    highp vec2 v0 = c - a;
    highp vec2 v1 = b - a;
    highp vec2 v2 = coord - a;
    
    highp float dot00 = dot(v0, v0);
    highp float dot01 = dot(v0, v1);
    highp float dot02 = dot(v0, v2);
    highp float dot11 = dot(v1, v1);
    highp float dot12 = dot(v1, v2);
    
    highp float deno = (dot00 * dot11 - dot01 * dot01);
    
    highp float u = (dot11 * dot02 - dot01 * dot12);
    highp float v = (dot00 * dot12 - dot01 * dot02);
    
    return u >= 0.0 && u <= deno && v >= 0.0 && v <= deno && u + v <= deno;
}

bool insideQuardrangle(highp vec2 coord, highp vec2 lt, highp vec2 lb, highp vec2 rt, highp vec2 rb)
{
    //return insideTriangle(coord, lt, lb, rt) || insideTriangle(coord, rb, lb, rt);
    return coord.x >= lt.x && coord.x <= rb.x && coord.y >= rb.y && coord.y <= lt.y;
}

void main()
{
    initBezier(0.0, 0.0, gradientFactor, 1.0);
    
    gl_FragColor = gradientColorBegin + (gradientColorEnd - gradientColorBegin) * valueBezier(gradientDir.x * TextCoordOut.s + gradientDir.y * TextCoordOut.t);
    
    gl_FragColor.a = alpha * gl_FragColor.a;
    
    gl_FragColor.a = gl_FragColor.a * float(TextCoordOut.x >= clipLeftTop.x && TextCoordOut.x <= clipRightBottom.x && TextCoordOut.y >= clipRightBottom.y && TextCoordOut.y <= clipLeftTop.y);
}