#version 300 es

// ui_2d_element_render.fsh

in highp vec4       colorVarying;
in highp vec2       texCoordOut;
in highp vec2       imgTexCoordOut;
in highp vec2       oldTexCoordOut;
in highp vec2       maskTexCoordOut;

uniform highp float alpha;

uniform sampler2D   texture0;
uniform sampler2D   textureOld;
uniform sampler2D   maskTexture;

uniform highp float blendingFactor;

uniform highp vec2  clipLeftTop;
uniform highp vec2  clipRightBottom;

uniform int         needStencil;
uniform sampler2D   stencilTex;
uniform highp vec2  stencilTexSize;
uniform int         renderIndexFirst;
uniform int         renderIndexLast;

uniform highp float mipmapLevel;

out highp vec4      g_FragColor;

void main()
{
    highp vec4 color = texture(texture0, imgTexCoordOut, mipmapLevel);
    highp vec4 oldColor = texture(textureOld, oldTexCoordOut, mipmapLevel);
    
    g_FragColor = color * float(color.a != 0.0) * blendingFactor + oldColor * float(oldColor.a != 0.0) * (1.0 - blendingFactor);
    
    g_FragColor.a = texture(maskTexture, maskTexCoordOut).a * g_FragColor.a;
    
    g_FragColor.a = alpha * g_FragColor.a;
    
    if (needStencil == 1)
    {
        highp vec2 coord;
        coord.x = gl_FragCoord.x / stencilTexSize.x;
        coord.y = gl_FragCoord.y / stencilTexSize.y;
        highp vec4 c = texture(stencilTex, coord, 0.0);
        
        int index = int(c.r * 100.0 + 0.5) + int(c.g * 100.0 + 0.5) * 100 + int(c.b * 100.0 + 0.5) * 10000;
        if (index < renderIndexFirst || index > renderIndexLast)
        {
            g_FragColor.a = 0.0;
        }
    }
    else
    {
        g_FragColor.a = g_FragColor.a * float(texCoordOut.x >= clipLeftTop.x && texCoordOut.x <= clipRightBottom.x && texCoordOut.y >= clipRightBottom.y && texCoordOut.y <= clipLeftTop.y);
    }
}